@@ -418,20 +418,23 @@ idle_do_action (gpointer data)
   GtkWidget *widget;
   GailExpander *gail_expander;
 
+  GDK_THREADS_ENTER ();
+
   gail_expander = GAIL_EXPANDER (data);
   gail_expander->action_idle_handler = 0;
 
   widget = GTK_ACCESSIBLE (gail_expander)->widget;
-  if (widget == NULL)
-    /*
-     * State is defunct
-     */
-    return FALSE;
-
-  if (!GTK_WIDGET_IS_SENSITIVE (widget) || !GTK_WIDGET_VISIBLE (widget))
-    return FALSE;
+  if (widget == NULL /* State is defunct */ ||
+      !GTK_WIDGET_IS_SENSITIVE (widget) || !GTK_WIDGET_VISIBLE (widget))
+    {
+      GDK_THREADS_LEAVE ();
+      return FALSE;
+    }
 
   gtk_widget_activate (widget);
+
+  GDK_THREADS_LEAVE ();
+
   return FALSE; 
 }
 
